@@ -29,6 +29,7 @@ Commands:
 
 Options:
   --host=<host>        OneP URL. Default is $EXO_HOST or m2.exosite.com
+  --port=<port>        OneP port. Default is $EXO_HOST or 80
   --httptimeout=<sec>  HTTP timeout [default: 60]
   --https              Enable HTTPS
   --debug              Show info like stack traces
@@ -68,7 +69,10 @@ try:
 except:
     from exoline import __version__
 
-DEFAULT_HOST='m2.exosite.com'
+DEFAULT_HOST = 'm2.exosite.com'
+DEFAULT_PORT = '80'
+DEFAULT_PORT_HTTPS = '443'
+
 cmd_doc = {
     'read':
         '''Read data from a resource.\n\nUsage:
@@ -82,6 +86,10 @@ Options:
     --selection=all|autowindow|givenwindow  downsample method [default: all]
     --format=csv|raw         output format [default: csv]
     --timeformat=unix|human  unix timestamp or human-readable? [default: human]
+    --chunkhours=<hours>     break read into multiple requests of length
+                             <hours>, printing data as it is received and
+                             ignoring --limit. Note that this requires start
+                             and end time be set.
     {{ helpoption }}
 
     If <rid> is omitted, reads all datasources and datarules under <cik>.
@@ -245,10 +253,13 @@ class ExoRPC():
     Raises exceptions on error and provides some reasonable defaults.'''
     def __init__(self,
             host=DEFAULT_HOST,
+            port=None,
             httptimeout=60,
             https=False,
             verbose=True):
-        port = '443' if https else '80'
+
+        if port is None:
+            port = DEFAULT_PORT_HTTPS if https else DEFAULT_PORT
         self.exo = onep.OnepV1(host=host,
                                port=port,
                                httptimeout=httptimeout,
@@ -373,14 +384,70 @@ class ExoRPC():
                  sort='asc',
                  starttime=None,
                  endtime=None,
-                 selection='all'):
-        '''Reads multiple rids and returns combined timestamped data like this:
-               [[12314, [1, 77, 'a']], [12315, [2, 78, None]]]
+                 selection='all',
+                 chunkhours=None):
+        '''Generates multiple rids and returns combined timestamped data like this:
+               [12314, [1, 77, 'a']
+               [12315, [2, 78, None]]
            Where 1, 77, 'a' is the order rids were passed, and None represents
            no data in that dataport for that timestamp.'''
         options = self._readoptions(limit, sort, starttime, endtime, selection)
-        responses = self._exomult(cik, [['read', rid, options] for rid in rids])
-        return self._combinereads(responses)
+
+        def _read(cik, rids, options):
+            responses = self._exomult(cik, [['read', rid, options] for rid in rids])
+            return self._combinereads(responses)
+
+        def _read_chunk(cik, rids, options, start, end):
+            chunkoptions = options.copy()
+            chunkoptions['starttime'] = start
+            chunkoptions['endtime'] = end
+            # read all points
+            chunkoptions['limit'] = end - start
+            return _read(cik, rids, chunkoptions)
+
+        if chunkhours is None:
+            for r in _read(cik, rids, options):
+                yield r
+        else:
+
+            # maximum # seconds we want to read in one request
+            # TODO: is there a clever way to calculate optimal read
+            # sizes automatially? What users want is 1.) no
+            # timeout/onep error and 2.) progress indication of some kind.
+            # I'd rather the user not have to figure out chunkhours
+            # themselves.
+            max_sec = 60 * 60 * int(chunkhours)
+
+            # TODO: figure out whether it's probably a large read by reading
+            # info, usage, or by some calculation.
+            is_large_read = True
+
+            # TODO: figure out earliest timestamp with data and adjust
+            # starttime if necessary.
+
+            if is_large_read:
+
+                if 'sort' in options and options['sort'] == 'desc':
+                    # descending
+                    for end in range(options['endtime'],
+                                     options['starttime'],
+                                     -max_sec):
+                        start = max(end - max_sec, options['starttime'])
+                        for r in _read_chunk(cik, rids, options, start, end):
+                            yield r
+                else:
+                    # ascending
+                    for start in range(options['starttime'],
+                                       options['endtime'],
+                                       max_sec):
+                        end = min(start + max_sec, options['endtime'])
+                        for r in _read_chunk(cik, rids, options, start, end):
+                            yield r
+
+            else:
+                # make a single request
+                for r in _read(cik, rids, options):
+                    yield r
 
     def write(self, cik, rid, value):
         isok, response = self.exo.write(cik, rid, value, {})
@@ -1164,19 +1231,24 @@ def read_cmd(er, cik, rids, args):
 
             time.sleep(sleep_seconds)
     else:
+        chunkhours = args['--chunkhours']
+        if chunkhours is not None and (start is None or end is None):
+            raise ExoException(
+                "--chunkhours requires --start and --end be set")
         for t, v in er.readmult(cik,
                                 rids,
                                 sort='desc',
                                 starttime=start,
                                 endtime=end,
-                                limit=limit):
+                                limit=limit,
+                                chunkhours=chunkhours):
             printline(t, v)
 
 def plain_print(arg):
     print(arg)
 
 def handle_args(cmd, args):
-    er = ExoRPC(host=args['--host'], https=args['--https'], httptimeout=args["--httptimeout"])
+    er = ExoRPC(host=args['--host'], port=args['--port'], https=args['--https'], httptimeout=args["--httptimeout"])
     cik = args['<cik>']
 
     def rid_or_alias(rid):
@@ -1409,6 +1481,8 @@ def cmd(argv=None, stdin=None, stdout=None, stderr=None):
     # substitute environment variables
     if args['--host'] is None:
         args['--host'] = os.environ.get('EXO_HOST', DEFAULT_HOST)
+    if args['--port'] is None:
+        args['--port'] = os.environ.get('EXO_PORT', None)
 
     try:
         handle_args(cmd, args)
@@ -1424,6 +1498,12 @@ def cmd(argv=None, stdin=None, stdout=None, stderr=None):
         # pyonep library call threw an exception on purpose
         sys.stderr.write("One Platform exception: {0}\r\n".format(ex))
         return 1
+    except pyonep.exceptions.JsonRPCRequestException as ex:
+        sys.stderr.write("JSON RPC Request Exception: {0}\r\n".format(ex))
+        return 1
+    except pyonep.exceptions.JsonRPCResponseException as ex:
+        sys.stderr.write("JSON RPC Response Exception: {0}\r\n".format(ex))
+        return 1
     except KeyboardInterrupt:
         if args['--debug']:
             raise
