@@ -40,7 +40,10 @@ def _cmd(argv, stdin):
     if True:
         log.debug(' '.join([str(a) for a in argv]))
         if stdin is not None:
-            log.debug('    stdin: ' + stdin)
+            display = stdin
+            if len(display) > 2000:
+                display = display[:1000] + '\n...\n' + display[-1000:]
+            log.debug('    stdin: ' + display)
     if type(stdin) is str:
         sio = StringIO.StringIO()
         sio.write(stdin)
@@ -428,13 +431,13 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         self.l("{0} {1}".format(lines, rids))
         self.assertTrue(lines == rids, 'listing after create')
         r = rpc('drop', client.cik(), '--all-children')
-        self.assertEqual(r.exitcode, 0, 'drop --all-children succeeded')
+        self.ok(r, 'drop --all-children succeeded')
         r = rpc('listing', client.cik(), '--type=dataport', '--plain')
-        self.assertTrue(len(r.stdout) == 0), 'no dataports after drop --all-children'
+        self.ok(r, 'no dataports after drop --all-children', match='')
         r = rpc('drop', self.portalcik, client.rid)
-        self.assertEqual(r.exitcode, 0, 'drop client succeeded')
+        self.ok(r, 'drop client succeeded')
         r = rpc('info', self.portalcik, client.rid)
-        self.assertTrue(r.exitcode != 0 and r.stderr.endswith('restricted'), 'client gone after drop')
+        self.notok(r, 'client gone after drop', match='.*restricted')
 
 
     def spark_test(self):
@@ -444,17 +447,14 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
             rpc('create', cik, '--type=dataport', '--format=integer', '--ridonly').stdout)
         rpc('record', cik, rid, '--interval={0}'.format(240), *['--value={0}'.format(x) for x in range(1, 6)])
         r = rpc('intervals', cik, rid, '--days=1')
-        m = re.match("[^ ] {59}\n4m", r.stdout)
-        self.assertTrue(m is not None, "equally spaced points")
+        self.ok(r, "equally spaced points", match="[^ ] {59}\n4m")
         rpc('flush', cik, rid)
         r = rpc('intervals', cik, rid, '--days=1')
-        self.assertTrue(r.exitcode == 0 and r.stdout == '', "no data should output nothing")
+        self.ok(r, "no data should output nothing", match="")
         r = rpc('record', cik, rid, '--value=-1,1', '--value=-62,2', '--value=-3662,3', '--value=-3723,4')
-        self.assertTrue(r.exitcode == 0, "record points")
+        self.ok(r, "record points")
         r = rpc('intervals', cik, rid, '--days=1')
-        self.l(u'stdout: {0} ({1})'.format(r.stdout, len(r.stdout)))
-        m = re.match("^[^ ] {58}[^ ]\n1m 1s +1h$", r.stdout)
-        self.assertTrue(m is not None, "three points, two intervals")
+        self.ok(r, "three points, two intervals", match="^[^ ] {58}[^ ]\n1m 1s +1h$")
 
     def _latest(self, cik, rid, val, msg):
         r = rpc('read', cik, rid, '--format=raw')
@@ -511,6 +511,9 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         dp2 = parse_metric('dataport', r)
         self.l("dp1: {0} dp2: {1}".format(dp1, dp2))
         # TODO: why does this not increase consistently?
+        # note that this measures seconds that the dataport existed, so time
+        # must pass for the value to go up.
+        time.sleep(5)
         self.assertTrue(dp2 > dp1, 'adding dataport added to dataport metric')
 
     def readmultiple_test(self):
@@ -548,11 +551,11 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         self.assertEqual(lines[2], '2013-07-20 02:40:07,a,,', 'three timestamps')
 
         r = rpc('read', '--start=2013-07-20T3:00:09', cik, *rids)
-        self.assertEqual(r.exitcode, 0, 'no data read succeeds')
-        self.assertTrue(r.stdout == '', 'no data')
+        self.ok(r, "no data read succeeds", match='')
 
         rids.reverse()
         r = rpc('read', '--start=2013-07-20T3:00:08', '--end=2013-07-20T3:00:08', cik, *rids)
+        self.ok(r, 'rid order reversed')
         self.assertTrue(r.stdout == '2013-07-20 03:00:08,0.3,3,', 'rid order reversed')
 
 
@@ -563,10 +566,10 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         std, label = (r.stderr, "stderr") if stderr else (r.stdout, "stdout")
         if search is not None:
             self.assertTrue(re.search(search, std, flags=re.MULTILINE) is not None,
-                msg + ' - search\n' + label + ':' + std + '\nsearch re: ' + search)
+                msg + ' - failed to find {0}\n'.format(search) + label + ':' + std + '\nsearch expression: ' + search)
         if match is not None:
             self.assertTrue(re.match(match, std, flags=re.MULTILINE) is not None,
-                msg + ' - match\n' + label + ':' + std + '\nmatch re: ' + match)
+                msg + ' - failed to match {0}\n'.format(match) + label + ':' + std + '\nmatch expression: ' + match)
 
     def notok(self, response, msg='', search=None, match=None):
         if response.exitcode == 0:
@@ -582,7 +585,7 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
 
 
     def copy_test(self):
-        '''Test copy and diff commands'''
+        '''Copy and diff commands'''
 
         cik = self.client.cik()
 
@@ -630,3 +633,40 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         self.ok(r, 'lookup dataport on inner cik copy', match=self.RE_RID)
         innercopydataportrid = r.stdout
         '''
+
+    def connection_test(self):
+        '''Connection settings'''
+        cik = self.client.cik()
+
+        r = rpc('--port=80', '--host=m2.exosite.com', 'info', cik)
+        self.ok(r, 'valid port and host at command line')
+
+        r = rpc('--https', 'info', cik)
+        self.ok(r, 'https connection')
+
+        r = rpc('--port=443', '--host=m2.exosite.com', '--https', 'info', cik)
+        self.ok(r, 'invalid port')
+
+        r = rpc('--port=88', 'info', cik)
+        self.notok(r, 'invalid port', match='JSON RPC Request Exception.*')
+
+        # TODO: test httptimeout
+        #rid = self._rid(
+        #    rpc('create', cik, '--type=dataport', '--format=string', '--ridonly').stdout)
+
+        #timeoutreadsize = 2000
+        #limit = '--limit={0}'.format(timeoutreadsize)
+        #timeout = '--httptimeout=120'
+        #smalltimeout = '--httptimeout=1'
+
+        #r = rpc(timeout, 'record', cik, rid, '--interval={0}'.format(60), '-', stdin='0123456789\n' * timeoutreadsize)
+        #self.ok(r, 'large record')
+
+        #r = rpc('read', cik, limit)
+        #self.ok(r, 'large read with standard timeout', match=r"(.*\n){" + str(timeoutreadsize) + "}")
+
+        #r = rpc(timeout, 'read', cik, limit)
+        #self.ok(r, 'large read with large timeout')
+
+        #r = rpc(smalltimeout, 'read', cik, limit)
+        #self.notok(r, 'large read with small timeout')
